#!/bin/sh
# (c) Copyright 1995, 1996  James H. Lowe, Jr.
#
#$Id: shak.sh_v 1.79 1996/05/27 12:45:35 jhl Exp $
#
#   October 1995
#
#   SHAK is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# STATIC CONFIGURATION PARAMETERS:
# -----------------------------------------

SHAK_CFG_DIR="/usr/etc/shak"
SHAK_CFG_DIR_LC="/etc/shak"

SHAK_USERLOGIN="$USER"
SHAK_REALID="`id -nur`"

	#  mt(1) defaults: the name of the mt command.
SHAK_MT="mt"

	#  The path used for remote backups.
SHAK_clientpath="/bin:/usr/bin:/sbin:/usr/local/bin"

	# The pid number of this shell program.  Used by other programs
	# in forming file names.
SHAK_PID=$$

	# The name of the getopt(1) program.
shak_getopt="/usr/bin/getopt"

	# The base name component of a tape index file.
shak_indexbase="INDEX_"

	# Global temporary filename name components.
shak_tmpdir="/usr/tmp"
shak_tmpbasename="${shak_tmpdir}/shak.${SHAK_PID}"

	# The directory where lock files will be written.
shak_lockdir="/var/lock"

	# The wake-up frequency (sec) and the total wait time.
	# This is used by a waiting backup.
shak_chkfreq=25
shak_waittime=14400
	
	# This is the default user that gets the notification mail
shak__mailuser="root"

	# Set this default fo notification mail
shak__mailflag="yes"

	# Only root can make a new tape, fix a tape.
	# Requires the variable USER be "root" and the real uid
	# must be root.
shak_rootonly0="no"

	# Internal debugging levels
	# set to 2 and yes for debugging.
shak__debugQ=1
shak__debug_locks=no

#######################################################################
#######################################################################
#######################################################################

export SHAK_MT
export shak_getopt
export shak_indexbase
export SHAK_clientpath
export SHAK_PID
export shak_tmpdir
export shak_tmpbasename
export shak_lockdir
export shak_chkfreq
export shak_waittime
export shak_rootonly0

	# Set the signal handler.
trap '${SHAK_LIBPATH}/shak_shutdown; exit $?' 1 2 15

	# make a empty temporary file.
TMPFILE_ENV=${shak_tmpbasename}.config0
umask 077
>$TMPFILE_ENV


	# Make a list of pre-existing configuration item settings.
env | egrep -e SHAK_SHAKPATH -e SHAK_USERPATH -e SHAK_TAPEHOST -e SHAK_TAPEDEVICE \
-e SHAK_INDEXPATH -e SHAK_LOGPATH -e SHAK_LOGBASENAME -e SHAK_REMSH -e SHAK_DOMAIN \
-e SHAK_LOGHOST -e SHAK_INDEXHOST -e SHAK_DEFAULTTEST -e SHAK_TAPECAPACITY -e SHAK_USEHEADERS |
sed -e 's@=.*@@' >>$TMPFILE_ENV


	# ** WatchDog Timer Defaults **
	#	Set to zero to disable watchdog timer.
	#--------------------------------------------------
# Time to perform any mt(1) tape operation
	delay__tape=4000
# Time to read a disk file
	delay__readfile=120
# Time to write a disk file
	delay__writefile=120
# Time to read a tape header file
	delay__readtape=120
# Time to write  to the tape, low value
	delay__writetape0=240
# Time to write  to the tape, high value
	delay__writetape1=2000
# Time to write a header to tape
	delay__writeheader=120
# Time to run the User backup
	delay__userbackup=0
	# ** Watch dog Variables **
	#	If set by parent shell of shak then the defaults 
	#	are not applied. 
	#------------------------------------------------------
delay_tape=${delay_tape-${delay__tape}}
export delay_tape
delay_readfile=${delay_readfile-${delay__readfile}}
export delay_readfile
delay_writefile=${delay_writefile-${delay__writefile}}
export delay_writefile
delay_readtape=${delay_readtape-${delay__readtape}}
export delay_readtape
delay_writetape0=${delay_writetape0-${delay__writetape0}}
export delay_writetape0
delay_writetape1=${delay_writetape1-${delay__writetape1}}
export delay_writetape1
delay_writeheader=${delay_writeheader-${delay__writeheader}}
export delay_writeheader
delay_userbackup=${delay_userbackup-${delay__userbackup}}
export delay_userbackup

	# Set the shak_mailuser variable.
shak_mailuser=${shak_mailuser-${shak__mailuser}}
export shak_mailuser

	# Set this to yes to enable notification mail
shak_mailflag=${shak_mailflag-${shak__mailflag}}
export shak_mailflag

shak_debugQ=${shak_debugQ-${shak__debugQ}}
export shak_debugQ

shak_debug_locks=${shak_debug_locks-${shak__debug_locks}}
export shak_debug_locks


#################################################################################
## END OF POLICY ITEMS ##########################################################
#################################################################################

	# The usage instructions.
shak_usage ()  {
          echo "Usage: shak [ options ] subcommand [args ...]" 1>&2 
          echo "shak [options...] <subcommand> [args ...]" 1>&2 
          echo "shak [options...] run  [-r stderrfile] [-h  host]  -t  type  file" 1>&2
          echo "shak [options...] testtape [-abmx] [ -f file] [-o file] [command]" 1>&2
          echo "shak [options...] copyheader [-f file]" 1>&2
          echo "shak [options...] newtape [-i tape_number] [-c]" 1>&2
	  echo "shak [options...] fix [-c|-l] [-m number] {-f|-t}" 1>&2
	  echo "shak [options...] query {-r|-s filespec }" 1>&2
	  echo "shak [options...] admin -o arg" 1>&2
}



	#  Set some defaults 
shak_async="0";
shak_rsh_is_sh="1";
shak_nouserconfig="0"
shak_quickrun="0"
shak_no_run_user="0"
shak_expert="0";
shak_trunclevel="0"
shak_debug="0"
shak_interactive="0"
shak_noindexfile="0"
shak_minimal="0"
userconfig=""
shak_noreadconfig="0"
shak_noheader="0"
shak_userenv="0"
shak_headeroverride="0"
shak_env="0"
shak_Q=0
shak_verifyi="0"
SHAK_TAPENUMBER=""
SHAK_tapenumber=""
SHAK_MTHOST=""
shak_batchmode=0
shak__batchmode=0
SHAK_TAPECAPACITY=""
SHAK_DTAPECAPACITY="0"


export SHAK_USERLOGIN
export SHAK_REALID
export SHAK_TAPENUMBER
export SHAK_tapenumber
export SHAK_TAPECAPACITY
export SHAK_DTAPECAPACITY
export shak_verifyi
export shak_batchmode
export shak__batchmode
export shak_debugQ



shak_defaults () {

SHAK_NULLDATE="1970/01/01 00:00:00 UTC"
SHAK_NULLTIME="0"
set ${SHAK_SHAKPATH="/usr/local/lib/shak"}
set ${SHAK_USERPATH="/usr/local/sbin/backup/shak"}
SHAK_HOST="localhost"
if [ -f "/bin/hostname" ]; then
   SHAK_HOST="`/bin/hostname | sed -e 's/^\([^\.]*\)\..*/\1/'`"
else
   SHAK_HOST="`hostname | sed -e 's/^\([^\.]*\)\..*/\1/'`"
fi

set ${SHAK_TAPEHOST="localhost"}
set ${SHAK_INDEXHOST="localhost"}
set ${SHAK_LOGHOST="localhost"}
set ${SHAK_TAPEDEVICE="/dev/nst0"}
set ${SHAK_REMSH="rsh"}
set ${SHAK_LOGPATH="/var/adm/backup/shak"}
set ${SHAK_LOGBASENAME="LOG_"}
set ${SHAK_MT_EOM="eom"}

# Default tape capacity
set ${SHAK_DTAPECAPACITY=0}

if [ ! "$SHAK_TAPECAPACITY" ]; then
	SHAK_TAPECAPACITY=$SHAK_DTAPECAPACITY
fi

if [ ! "$SHAK_TAPENUMBER" ]; then
	SHAK_TAPENUMBER="NULL"
fi

SHAK_SHAKDATE="${SHAK_NULLDATE}"
SHAK_TIME_T="${SHAK_NULLTIME}"
SHAK_SHAKTYPE="NULL"

set ${SHAK_INDEXPATH="/var/adm/backup/shak"}
SHAK_LIBPATH="${SHAK_SHAKPATH}/lib"
SHAK_LOGNAME="${SHAK_LOGPATH}/${SHAK_LOGBASENAME}"
set ${SHAK_DOMAIN="domainname"}
SHAK_OVERWRITE="0"
set ${SHAK_DEFAULTTEST="-b"}
set ${SHAK_USEHEADERS=yes}


export SHAK_DOMAIN
export SHAK_TAPENUMBER
export SHAK_LOGPATH
}


shak_configure () {
TMPFILE=$2
umask 077
>$TMPFILE
TMPFILE_ENV=$3
CONFIGFILE="$1"

       egrep -e "^#shakpath"\|"^shakpath" \
             -e "^#userpath"\|"^userpath" \
             -e "^#tapehost"\|"^tapehost" \
             -e "^#device"\|"^device" \
             -e "^#indexpath"\|"^indexpath" \
             -e "^#logpath"\|"^logpath" \
             -e "^#logbasename"\|"^logbasename" \
             -e "^#remsh"\|"^remsh" \
             -e "^#domain"\|"^domain" \
             -e "^#loghost"\|"^loghost" \
             -e "^#indexhost"\|"^indexhost" \
             -e "^#defaulttest"\|"^defaulttest" \
             -e "^#tapesecure"\|"^tapesecure" \
             -e "^#sitesecure"\|"^sitesecure" \
             -e "^#devicesecure"\|"^devicesecure" \
             -e "^#tapecapacity"\|"^tapecapacity" \
             -e "^#mt_eom"\|"^mt_eom" \
             -e "^#useheaders"\|"^useheaders"  < ${CONFIGFILE} |

       sed -e 's/^#//' |
       sed \
       -e 's/shakpath:/SHAK_SHAKPATH=/' \
       -e 's/userpath:/SHAK_USERPATH=/' \
       -e 's/tapehost:/SHAK_TAPEHOST=/' \
       -e 's/device:/SHAK_TAPEDEVICE=/' \
       -e 's/indexpath:/SHAK_INDEXPATH=/' \
       -e 's/logpath:/SHAK_LOGPATH=/' \
       -e 's/logbasename:/SHAK_LOGBASENAME=/' \
       -e 's/remsh:/SHAK_REMSH=/' \
       -e 's/domain:/SHAK_DOMAIN=/' \
       -e 's/loghost:/SHAK_LOGHOST=/' \
       -e 's/indexhost:/SHAK_INDEXHOST=/' \
       -e 's/defaulttest:/SHAK_DEFAULTTEST=/' \
       -e 's/tapecapacity:/SHAK_TAPECAPACITY=/' \
       -e 's/sitesecure:/SHAK_sitesecure=/' \
       -e 's/devicesecure:/SHAK_devicesecure=/' \
       -e 's/tapesecure:/SHAK_tapesecure=/' \
       -e 's/mt_eom:/SHAK_MT_EOM=/' \
       -e 's/useheaders:/SHAK_USEHEADERS=/' | 
        
	(
	if [ $shak_env = "1" -o "$SHAK_sitesecure" = "yes" ]; then
		fgrep  -w -v -f $TMPFILE_ENV
        else
		cat
	fi
	)	>>$TMPFILE	
	
	. $TMPFILE
        echo SHAK_LIBPATH=$SHAK_SHAKPATH/lib >>$TMPFILE
return
}


PROG=$0



# separate the shak args from the subprogram args
# in order to keep as much as possible away from getopt
# because the call strips the quotes. Hopefully the 
# shak args are simple enough where nothing will have to
# quoted.

shakargs=""
until [ $# = 0 ];
do
	case $1 in
	      run|newtape|copyheader|testtape|fix|query) 
		  break
		  ;;

              *) shakargs="$shakargs $1"
	         shift
		 ;;
        esac
done


# Run getopts
TMPFILEOPT=${shak_tmpbasename}.getopts$$
aftergetopt="`$shak_getopt i:ac:dEg:hk:lmNqQrtTuUvwxy $shakargs 2>$TMPFILEOPT`"

set -- $aftergetopt "$@"


# Check for getopts failure.
if [ $? != 0 ]; then
   cat <$TMPFILEOPT
   /bin/rm -f $TMPFILEOPT
   shak_usage 
   exit 1
fi
grep '.' <$TMPFILEOPT
if [ "$?" = "0" ]; then
   cat <$TMPFILEOPT
   /bin/rm -f $TMPFILEOPT
   shak_usage
   exit 1
fi
/bin/rm -f $TMPFILEOPT



# Parse options
for i in $*; do
   case $1 in
     -a)
        shak_async="1";
	shift;
	;;
     -c)
	shift;
        if [ $# -ge 1 ]; then
           SHAK_TAPECAPACITY="$1"
           export SHAK_TAPECAPACITY
	   shift
	else
	  usage
	fi
	;;
     -d)
        shak_nouserconfig="1";
	shift;
	;;
     -E)
        shak_env="1";
	shift;
	;;
     -h)
        shak_rsh_is_sh="0";
	shift;
        ;;
     -i)
	shift;
        if [ $# -ge 1 ]; then
           SHAK_TAPENUMBER="`echo $1 | awk '{ printf ("%04s\n",$1) }'`"
	   SHAK_tapenumber=$SHAK_TAPENUMBER
           export SHAK_TAPENUMBER
           export SHAK_tapenumber
	   
	   shift
	else
	  usage
	fi
	;;
     -g)
	shift;
        if [ "$1" = "-" ]; then 
     	  exit 1
        else
	  userconfig="$1";
	fi
        shift;
	;;
     -k)
	shift;
        if [ "$1" = "-" ]; then 
     	  exit 1
        else
	  shak_trunclevel="$1";
	fi
        shift;
	;;
     -l)
        shak_noreadconfig="` expr $shak_noreadconfig + 1 `"
	shift;
	;;
     -N)
        shak_verifyi="1"
        shift;
        ;;
     -q)
        shak_quickrun="1"
        shift;
        ;;
     -Q)
        shak_noheader=2
        shak_noindexfile=1
	shak_Q=1
        shift;
        ;;
     -r)
        shak_userenv="1"
        shift;
        ;;
     -t)
        shak_noindexfile="1"
        shift;
        ;;
     -T)
        shak_noheader=1
        shift;
        ;;
     -v)
        shak_debug="` expr $shak_debug + 1 `"
        shift;
        ;; 
     -w)
	shift;
        if [ $# -ge 1 ]; then
           shak_waittime=$1
           export shak_waittime
	   shift
	else
	  usage
	fi
	;;
     -u)
        shak_interactive="1"
        shift;
        ;; 
      
     -U)
        shak_headeroverride="1"
        shift;
        ;; 
     
     -m)
        shak_minimal="1"
        shift;
        ;; 
     -x)
        shak_expert="1";
	shift;
	;;
     -y)
        shak_minimal="1"
        shak_no_run_user="` expr $shak_no_run_user + 1 `"
        shift;
        ;;
     --) shift;
         break;
	 ;;
    esac
done




# Expert mode incompatible with query modes.
if [ $shak_expert -eq  1 ]; then
	if [ $shak_Q -eq 1 -o $shak_no_run_user -ge 1 ]; then
		echo "shak: incompatible options: x with Q or y." 1>&2
		exit 1
	fi
fi



# Query modes are mutually exclusive.
if [ $shak_Q -eq 1 -a $shak_no_run_user -ge 1 ]; then
	echo "shak: incompatible options: Q with y." 1>&2
	exit 1
fi



# If not reading any config files then set some defaults
# else run the standards defaults setup function.
if [ "$shak_noreadconfig" -gt "1" ]; then
	SHAK_NULLDATE="1970/01/01 00:00:00 UTC"
	SHAK_NULLTIME="0"
	SHAK_HOST="localhost"

	SHAK_SHAKDATE="${SHAK_NULLDATE}"
	SHAK_TIME_T="${SHAK_NULLTIME}"

	if [ ! "$SHAK_TAPENUMBER" ]; then
		SHAK_TAPENUMBER="NULL"
	fi

	if [ ! "$SHAK_TAPECAPACITY" ]; then
		SHAK_TAPECAPACITY=$SHAK_DTAPECAPACITY
	fi
	
	if [ ! "$SHAK_MTHOST" ]; then
		SHAK_MTHOST="$SHAK_TAPEHOST"
	fi

	SHAK_OVERWRITE="0"
	SHAK_LIBPATH="${SHAK_SHAKPATH}/lib"
	SHAK_LOGNAME="${SHAK_LOGPATH}/${SHAK_LOGBASENAME}"
	export SHAK_LIBPATH
	export SHAK_LOGNAME
	export SHAK_DOMAIN
	export SHAK_TAPENUMBER
else
	shak_defaults
fi

export shak_minimal
export shak_debug
export shak_interactive
export shak_quickrun
export shak_noreadconfig
export shak_no_run_user
export shak_nouserconfig
export shak_async
export shak_expert
export shak_rsh_is_sh
export shak_trunclevel
export shak_noindexfile
export shak_noheader
export shak_userenv
export shak_env
export shak_Q
export shak_verifyi



# Comply with verbose execution request.
if [ "$shak_debug" -ge 2 ]; then
 set -vx
fi


# get a tmpfile name 
TMPFILE="${shak_tmpbasename}.shak$$"



# Save the arguments.  This is an attempt to be
# bourne (non-POSIX) shell compatible.
#nargs=$#
#ARGS=""
#while [ $nargs -ge 1 ];
# do
#   DOL1="`echo "$1" | sed -e 's|:|\\\072|'`"
#   ARGS="$ARGS${DOL1}:"
#   shift
#   nargs="`expr $nargs - 1`"
# done
#OLDIFS="$IFS"



# Read the configuration files.
if [ $shak_noreadconfig -ge "1" ]; then
   if [ "$userconfig" ]; then
     if [ -f "$userconfig" ]; then
       shak_configure $userconfig $TMPFILE $TMPFILE_ENV
       . $TMPFILE 
       /bin/rm -f $TMPFILE
     else
       echo "shak: $userconfig not found." 1>&2  
       exit 1
     fi
   fi
else
   if [ -f ${SHAK_CFG_DIR}/shak.conf ]; then
     shak_configure ${SHAK_CFG_DIR}/shak.conf $TMPFILE $TMPFILE_ENV
     . $TMPFILE 
     /bin/rm -f $TMPFILE
   fi
   if [ -f ${SHAK_CFG_DIR_LC}/shak.conf ]; then
     if [ "$SHAK_sitesecure" = "yes" ]; then
        . ${SHAK_LIBPATH}/shak_export
        umask 077
        >$TMPFILE_ENV
	env | egrep -e SHAK_SHAKPATH -e SHAK_USERPATH -e SHAK_TAPEHOST -e SHAK_TAPEDEVICE \
	-e SHAK_INDEXPATH -e SHAK_LOGPATH -e SHAK_LOGBASENAME -e SHAK_REMSH -e SHAK_DOMAIN \
	-e SHAK_LOGHOST -e SHAK_INDEXHOST -e SHAK_DEFAULTTEST -e SHAK_TAPECAPACITY -e SHAK_USEHEADERS \
	-e SHAK_sitesecure -e SHAK_tapesecure -e SHAK_devicesecure |
	sed -e 's@=.*@@' >>$TMPFILE_ENV
     fi
     shak_configure ${SHAK_CFG_DIR_LC}/shak.conf $TMPFILE $TMPFILE_ENV
     . $TMPFILE 
     /bin/rm -f $TMPFILE
   fi
   if [ "$userconfig" ]; then
     if [ -f "$userconfig" ]; then
       shak_configure $userconfig $TMPFILE $TMPFILE_ENV
       . $TMPFILE 
       /bin/rm -f $TMPFILE
     else
       echo "shak: $userconfig not found." 1>&2  
       exit 1
     fi
   fi
fi

/bin/rm -f "$TMPFILE_ENV"


if [ "$SHAK_TAPEHOST" = localhost ]; then
	if [ -f "/bin/hostname" ]; then
		SHAK_TAPEHOST="`/bin/hostname | sed -e 's/^\([^\.]*\)\..*/\1/'`"
	else
		SHAK_TAPEHOST="`hostname | sed -e 's/^\([^\.]*\)\..*/\1/'`"
	fi
fi


# Set up some implied states.
if [ $shak_headeroverride -gt 0 ]; then
	SHAK_USEHEADERS=yes
fi

if [ $SHAK_USEHEADERS = "no" ]; then
	shak_noheader=1
fi

if [ $shak_noheader -gt 0 ]; then
	SHAK_USEHEADERS=no
fi

if [ $shak_Q -eq 1 ]; then
	SHAK_TAPEDEVICE=/dev/null
fi

if [ $SHAK_tapesecure = "yes" ]; then
	shak_verifyi=1
	SHAK_TAPENUMBER="NULL"
fi



# export the variables.
. ${SHAK_LIBPATH}/shak_export



# Restore the arguments. Function calling is done for now.
#IFS=":"
#if [ "$ARGS" ]; then
#  set - $ARGS
#fi
#IFS="$OLDIFS"
#
#if [ $shak_debug -eq 1 ]; then
#  ${SHAK_LIBPATH}/shak_debug
#fi



rrun=0
export rrun
# Main Case Statement.
# These are the things that shak does.
case $1 in
    admin)
          shift;
	  ${SHAK_LIBPATH}/shak_admin "$@"
	  retval=$?
	  ;;
    run)  
	  shift;
	  rrun=1
	  ${SHAK_LIBPATH}/shak_run "$@"
	  retval="$?"
          ;;

    testtape)
	  shift;

	  if [ "$shak_no_run_user" -gt 1 -o "$shak_Q" -eq 1 ]; then
		:
	  else
		${SHAK_LIBPATH}/shak_waitlock $shak_waittime
		retval=$?
		if [ $retval -eq 0 ]; then
			:
		else
			exit $retval
		fi
	  fi
	  $SHAK_LIBPATH/shak_testtape "$@"
	  retval="$?"
          ;;
    newtape)
	  shift; 
	  SHAK_useheader=$SHAK_USEHEADERS
	  export SHAK_useheader
          SHAKDAT="${SHAK_LIBPATH}/shak0_date"
	  sh ${SHAK_LIBPATH}/shak_newtape "$@" "`${SHAKDAT}`"
          retval=$?
          ;;	  
    copyheader)
	  shift;
          sh $SHAK_LIBPATH/shak_copyheader "$@"
	  retval="$?"
          ;;
    fix)
	  shift;
          ${SHAK_LIBPATH}/shak_fixtape "$@"
          retval=$?
          ;;

    query)
          shift;
	  ${SHAK_LIBPATH}/shak_query "$@"
	  retval=$?
	  ;;

    *)
	  shak_usage
          exit 2
    esac

	if [ $retval -ne 0 -a $rrun = 1 ]; then
		sleep 5
		# This to avoid a close finish (race) with
		# the operations of shak_haltq
	fi

	( trap '' 1 2 15; "${SHAK_LIBPATH}/shak_unlock" ) &
	rm -f /usr/tmp/shak.${SHAK_PID}*

exit $retval

